@@ -18,7 +18,7 @@
 
 
 import enum
-from typing import List, NamedTuple, Optional, Tuple
+from typing import List, NamedTuple, Optional, Tuple, Set, FrozenSet
 import unittest
 
 from typedload import dataloader, load, dump, typechecks, exceptions
@@ -107,3 +107,23 @@ class TestExceptionsStr(unittest.TestCase):
             except Exception as e:
                 str(e)
 
+    def test_nested_wrong_type(self):
+        with self.assertRaises(exceptions.TypedloadException):
+            load([[1]], List[List[bytes]])
+        with self.assertRaises(exceptions.TypedloadException):
+            load([[1]], List[Tuple[bytes, ...]])
+        with self.assertRaises(exceptions.TypedloadException):
+            load([[1]], List[Set[bytes]])
+        with self.assertRaises(exceptions.TypedloadException):
+            load([[1]], List[FrozenSet[bytes]])
+
+    def test_notiterable_exception(self):
+        loader = dataloader.Loader()
+        with self.assertRaises(exceptions.TypedloadTypeError):
+            loader.load(None, List[int])
+        with self.assertRaises(exceptions.TypedloadTypeError):
+            loader.load(None, Tuple[int, ...])
+        with self.assertRaises(exceptions.TypedloadTypeError):
+            loader.load(None, Set[int])
+        with self.assertRaises(exceptions.TypedloadTypeError):
+            loader.load(None, FrozenSet[int])
