@@ -321,7 +321,7 @@ def _literalload(l: Loader, value: Any, type_) -> Any:
     Checks if the value is within the allowed literals and
     returns it.
     """
-    if value in literalvalues(type_):
+    if value in type_.__args__:
         return value
     raise TypedloadValueError('Not one of the allowed values in %s' % tname(type_), value=value, type_=type_)
 
@@ -409,10 +409,7 @@ def _tupleload(l: Loader, value: Any, type_) -> Tuple:
     """
     This loads into something like Tuple[int,str]
     """
-    if HAS_TUPLEARGS:
-        args = type_.__args__
-    else:
-        args = type_.__tuple_params__
+    args = type_.__args__
 
     if len(args) == 2 and args[1] == ...: # Tuple[something, ...]
         return _iterload(l, value, type_, tuple)
@@ -744,12 +741,28 @@ def _datetimeload(l: Loader, value: Any, type_) -> Union[datetime.date, datetime
         raise TypedloadTypeError(str(e), type_=type_, value=value)
 
 
+def _get_attr_converter_type(c: "Callable"):
+    """
+    c is a converter function passed to an attr field
+
+    it is supposed to have 1 parameter only
+
+    If it's typed, return the type of the parameter. Otherwise return Any
+    """
+    hints = get_type_hints(c)
+    if len(hints) > 0 and len(hints) <= 2:
+        if 'return' in hints:
+            del hints['return']
+        return next(iter(hints.values()))
+    return Any
+
+
 def _attrload(l: Loader, value: Any, type_) -> Any:
     from attr._make import _Nothing as NOTHING
 
     fields = {i.name for i in type_.__attrs_attrs__}
     necessary_fields = set()
-    type_hints = {i.name: i.type for i in type_.__attrs_attrs__}
+    type_hints = {i.name: (_get_attr_converter_type(i.converter) if i.converter else i.type) for i in type_.__attrs_attrs__}
     namesmap = {}  # type: Dict[str, str]
 
     for attribute in type_.__attrs_attrs__:
